package de.intevation.flys.importer;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.exception.ConstraintViolationException;

import de.intevation.flys.model.River;
import de.intevation.flys.model.SQRelation;
import de.intevation.flys.model.TimeInterval;


public class ImportSQRelation {

    private static Logger log = Logger.getLogger(ImportSQRelation.class);

    private ImportTimeInterval timeInterval;

    private String description;

    private List<ImportSQRelationValue> values;

    private SQRelation peer;

    public ImportSQRelation() {
        this.values = new ArrayList<ImportSQRelationValue>();
    }

    public void storeDependencies(River river) {
        log.info("store dependencies");

        SQRelation peer = getPeer(river);

        timeInterval.getPeer();

        if (peer != null) {
            int count = 0;

            for (ImportSQRelationValue value : values) {
                try {
                    value.storeDependencies(peer);
                    count++;
                }
                catch (SQLException sqle) {
                    log.warn("ISQ: Unable to store sq relation value.", sqle);
                }
                catch (ConstraintViolationException cve) {
                    log.warn("ISQ: Unable to store sq relation value.", cve);
                }
            }

            log.info("stored " + count + " sq relation values.");
        }
    }

    public SQRelation getPeer(River river) {
        log.debug("getPeer()");

        if (peer == null) {
            TimeInterval timeInter = timeInterval.getPeer();

            if (timeInter == null) {
                log.warn("ISQ: Cannot determine sq relation without time interval");
                return null;
            }

            Session session = ImporterSession.getInstance()
                .getDatabaseSession();

            Query query = session
                .createQuery("FROM SQRelation WHERE river=:river AND timeInterval=:timeInter");

            query.setParameter("river", river);
            query.setParameter("timeInter", timeInter);

            List<SQRelation> sq = query.list();

            if (sq.isEmpty()) {
                log.info("create new SQ relation '" + description + "'");

                peer = new SQRelation(river, timeInter, description);
                session.save(peer);
            }
            else {
                peer = sq.get(0);
            }
        }

        return peer;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setTimeInterval(ImportTimeInterval timeInterval) {
        this.timeInterval = timeInterval;
    }

    public void addValue(ImportSQRelationValue value) {
        if (value != null) {
            this.values.add(value);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
